package cn.minbb.tester.activities;

import android.annotation.SuppressLint;
import android.app.ProgressDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.design.widget.FloatingActionButton;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.app.AppCompatDelegate;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Toast;

import com.alibaba.fastjson.JSONObject;
import com.joanzapata.iconify.widget.IconTextView;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import cn.minbb.tester.R;
import cn.minbb.tester.adapter.BasicDeviceAdapter;
import cn.minbb.tester.ctrl.App;
import cn.minbb.tester.ctrl.BluetoothChatService;
import cn.minbb.tester.utils.DataTranslation;

public class BasicDeviceActivity extends AppCompatActivity implements View.OnTouchListener {

    @BindView(R.id.noDevice)
    IconTextView noDevice;
    @BindView(R.id.content)
    RecyclerView contentRecyclerView;
    @BindView(R.id.opUp)
    FloatingActionButton opUp;
    @BindView(R.id.opLeft)
    FloatingActionButton opLeft;
    @BindView(R.id.opCenter)
    FloatingActionButton opCenter;
    @BindView(R.id.opRight)
    FloatingActionButton opRight;
    @BindView(R.id.opDown)
    FloatingActionButton opDown;

    private ProgressDialog dialog;

    private BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
    private BluetoothChatService chatService = null;

    private BasicDeviceAdapter basicDeviceAdapter;
    private List<BasicDeviceAdapter.ItemBean> itemBeanList = new ArrayList<>();
    // 输出流缓冲区
    private StringBuffer outStringBuffer = new StringBuffer("");

    static {
        AppCompatDelegate.setCompatVectorFromResourcesEnabled(true);
    }

    @SuppressLint("ClickableViewAccessibility")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_basic_device);
        ButterKnife.bind(this);

        // 设置广播信息过滤
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(BluetoothDevice.ACTION_FOUND);
        intentFilter.addAction(BluetoothDevice.ACTION_BOND_STATE_CHANGED);
        intentFilter.addAction(BluetoothAdapter.ACTION_SCAN_MODE_CHANGED);
        intentFilter.addAction(BluetoothAdapter.ACTION_STATE_CHANGED);
        // 注册广播接收器，接收并处理搜索结果
        this.registerReceiver(receiver, intentFilter);

        App.setupActionBar(this);
        noDevice.setVisibility(View.VISIBLE);

        basicDeviceAdapter = new BasicDeviceAdapter(this, itemBeanList);
        contentRecyclerView.setAdapter(basicDeviceAdapter);
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(this);
        linearLayoutManager.setOrientation(LinearLayoutManager.VERTICAL);
        linearLayoutManager.setReverseLayout(false);
        contentRecyclerView.setLayoutManager(linearLayoutManager);
        opUp.setOnTouchListener(this);
        opLeft.setOnTouchListener(this);
        opCenter.setOnTouchListener(this);
        opRight.setOnTouchListener(this);
        opDown.setOnTouchListener(this);
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (chatService != null) {
            // 只有状态是 STATE_NONE，我们知道我们还没有启动蓝牙
            if (chatService.getState() == BluetoothChatService.STATE_NONE) {
                // 启动 BluetoothChat 服务
                chatService.start();
            }
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mHandler.removeCallbacksAndMessages(null);
        // 停止蓝牙通信连接服务
        if (chatService != null) chatService.stop();
    }

    public static void start(Context context) {
        Intent starter = new Intent(context, BasicDeviceActivity.class);
        context.startActivity(starter);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        menu.add(0, 0, 0, "断开连接").setIcon(R.drawable.ic_close_white_24dp).setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                if (chatService != null && chatService.getState() == BluetoothChatService.STATE_CONNECTED) {
                    AlertDialog.Builder builderQuit = new AlertDialog.Builder(BasicDeviceActivity.this);
                    builderQuit.setTitle("退出")
                            .setMessage("返回将断开设备的蓝牙连接")
                            .setNegativeButton("取消", null)
                            .setPositiveButton("退出", (dialog, which) -> {
                                chatService.stop();
                                finish();
                            })
                            .create()
                            .show();
                } else {
                    finish();
                }
                break;
            case 0:
                if (chatService != null && chatService.getState() == BluetoothChatService.STATE_CONNECTED) {
                    AlertDialog.Builder builderClose = new AlertDialog.Builder(BasicDeviceActivity.this);
                    builderClose.setTitle("断开连接")
                            .setMessage("确定断开设备的蓝牙连接吗？")
                            .setNegativeButton("取消", null)
                            .setPositiveButton("确定", (dialog, which) -> {
                                if (null != chatService) chatService.stop();
                            })
                            .create()
                            .show();
                } else {
                    Toast.makeText(getApplicationContext(), "设备未连接", Toast.LENGTH_SHORT).show();
                }
                break;
            default:
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        switch (requestCode) {
            case 0:
                if (resultCode == RESULT_OK) {
                    // 初始化蓝牙连接服务
                    chatService = new BluetoothChatService(this, mHandler);
                    // 获取设备的 MAC 地址
                    String address = data.getExtras().getString(BluetoothChatService.EXTRA_DEVICE_ADDRESS);
                    // 尝试连接到设备
                    chatService.connect(bluetoothAdapter.getRemoteDevice(address));
                    // 初始化文本
                }
        }
    }

    @OnClick({R.id.noDevice})
    public void onViewClicked(View view) {
        switch (view.getId()) {
            case R.id.noDevice:
                startActivityForResult(new Intent(BasicDeviceActivity.this, ScanDeviceActivity.class), 0);
                break;
            default:
        }
    }

    @Override
    public boolean onTouch(View view, MotionEvent motionEvent) {
        switch (view.getId()) {
            case R.id.opUp:
                switch (motionEvent.getAction()) {
                    case MotionEvent.ACTION_DOWN:
                        sendDataToDevice("UD\r\n", false);
                        break;
                    case MotionEvent.ACTION_UP:
                        sendDataToDevice("UU\r\n", false);
                        break;
                }
                break;
            case R.id.opLeft:
                switch (motionEvent.getAction()) {
                    case MotionEvent.ACTION_DOWN:
                        sendDataToDevice("LD\r\n", false);
                        break;
                    case MotionEvent.ACTION_UP:
                        sendDataToDevice("LU\r\n", false);
                        break;
                }
                break;
            case R.id.opCenter:
                switch (motionEvent.getAction()) {
                    case MotionEvent.ACTION_DOWN:
                        sendDataToDevice("SD\r\n", false);
                        break;
                    case MotionEvent.ACTION_UP:
                        sendDataToDevice("SU\r\n", false);
                        break;
                }
                break;
            case R.id.opRight:
                switch (motionEvent.getAction()) {
                    case MotionEvent.ACTION_DOWN:
                        sendDataToDevice("RD\r\n", false);
                        break;
                    case MotionEvent.ACTION_UP:
                        sendDataToDevice("RU\r\n", false);
                        break;
                }
                break;
            case R.id.opDown:
                switch (motionEvent.getAction()) {
                    case MotionEvent.ACTION_DOWN:
                        sendDataToDevice("DD\r\n", false);
                        break;
                    case MotionEvent.ACTION_UP:
                        sendDataToDevice("DU\r\n", false);
                        break;
                }
                break;
            default:
        }
        return true;
    }

    private void showConnectDeviceProgress(boolean show) {
        if (dialog == null) {
            dialog = new ProgressDialog(this);
            dialog.setTitle("连接");
            dialog.setMessage("设备连接中······");
            dialog.setCancelable(false);
        }
        if (show) {
            dialog.show();
        } else {
            dialog.dismiss();
            dialog = null;
        }
    }

    private void sendDataToDevice(String data, boolean hex) {
        // 确保已连接
        if (null == chatService || chatService.getState() != BluetoothChatService.STATE_CONNECTED) {
            Toast.makeText(this, "设备未连接", Toast.LENGTH_SHORT).show();
            return;
        }
        // 检测是否有字符串发送
        if (data.length() > 0) {
            // 获取字符串并告诉 BluetoothChatService 发送
            if (hex) {
                chatService.write(DataTranslation.hexString2Bytes(data));
            } else {
                chatService.write(data.getBytes());
            }
            // 清空输出缓冲区
            outStringBuffer.setLength(0);
        } else {
            Toast.makeText(this, "发送内容不能为空", Toast.LENGTH_SHORT).show();
        }
    }

    @SuppressLint("HandlerLeak")
    private final Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case BluetoothChatService.MESSAGE_STATE_CHANGE:
                    System.err.println("MESSAGE_STATE_CHANGE = " + msg.arg1);
                    switch (msg.arg1) {
                        case BluetoothChatService.STATE_CONNECTED:
                            // 已连接
                            showConnectDeviceProgress(false);
                            noDevice.setVisibility(View.GONE);
                            break;
                        case BluetoothChatService.STATE_CONNECTING:
                            showConnectDeviceProgress(true);
                            break;
                        case BluetoothChatService.STATE_LISTEN:
                            showConnectDeviceProgress(false);
                            break;
                        case BluetoothChatService.STATE_NONE:
                            // 无连接
                            showConnectDeviceProgress(false);
                            noDevice.setVisibility(View.VISIBLE);
                            break;
                    }
                    break;
                case BluetoothChatService.MESSAGE_READ:
                    byte[] readBuf = (byte[]) msg.obj;
                    String readMessage = null;
                    try {
                        readMessage = new String(readBuf, 0, msg.arg1, "GBK");
                    } catch (UnsupportedEncodingException e) {
                        e.printStackTrace();
                    }
                    if (null != readMessage) {
                        System.err.println("===" + readMessage);
                        Pattern p = Pattern.compile("\\{.*?\\}");
                        Matcher m = p.matcher(readMessage);
                        // 判断正则表达式是否匹配到
                        if (m.find()) {
                            String data = m.group();
                            JSONObject jsonObject = JSONObject.parseObject(data);
                            itemBeanList.clear();
                            for (String key : jsonObject.keySet()) {
                                itemBeanList.add(new BasicDeviceAdapter.ItemBean(key, jsonObject.getString(key)));
                            }
                            basicDeviceAdapter.notifyDataSetChanged();
                        } else {
                            System.out.println("未匹配到数据");
                        }
                    }
                    break;
                case BluetoothChatService.MESSAGE_DEVICE_NAME:
                    String connectedDeviceName = msg.getData().getString(BluetoothChatService.DEVICE_NAME);
                    Toast.makeText(getApplicationContext(), "连接到 " + connectedDeviceName, Toast.LENGTH_SHORT).show();
                    break;
                case BluetoothChatService.MESSAGE_TOAST:
                    Toast.makeText(getApplicationContext(), msg.getData().getString(BluetoothChatService.TOAST), Toast.LENGTH_SHORT).show();
                    break;
                default:
            }
        }
    };

    private BroadcastReceiver receiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            if (BluetoothAdapter.ACTION_STATE_CHANGED.equals(action)) {
                new Handler().postDelayed(() -> runOnUiThread(() -> {
                    if (!bluetoothAdapter.isEnabled()) {
                        if (null != chatService) chatService.stop();
                    }
                }), 500);
            }
        }
    };
}
