package cn.minbb.tester.activities;

import android.annotation.SuppressLint;
import android.app.ProgressDialog;
import android.bluetooth.BluetoothAdapter;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.annotation.NonNull;
import android.support.design.widget.BottomNavigationView;
import android.support.v4.view.ViewPager;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.app.AppCompatDelegate;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.StaggeredGridLayoutManager;
import android.support.v7.widget.Toolbar;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import com.joanzapata.iconify.widget.IconTextView;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import cn.minbb.tester.R;
import cn.minbb.tester.adapter.MainPagerAdapter;
import cn.minbb.tester.adapter.WaterfallStaggeredAdapter;
import cn.minbb.tester.ctrl.BluetoothChatService;

public class MainActivity extends AppCompatActivity implements
        BottomNavigationView.OnNavigationItemSelectedListener,
        WaterfallStaggeredAdapter.OnItemClickListener {

    @BindView(R.id.toolbar)
    Toolbar toolbar;
    @BindView(R.id.viewPager)
    ViewPager viewPager;
    @BindView(R.id.navigation)
    BottomNavigationView navigation;
    @BindView(R.id.icon)
    IconTextView icon;
    @BindView(R.id.title)
    TextView title;
    @BindView(R.id.device)
    TextView device;

    private RecyclerView mainTab_1_recycle;

    private List<View> views = new ArrayList<>(3);
    private ProgressDialog dialog;

    private BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
    private BluetoothChatService chatService = null;

    static {
        AppCompatDelegate.setCompatVectorFromResourcesEnabled(true);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        ButterKnife.bind(this);

        initView();
        setListener();
        loadData();
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (chatService != null) {
            // 只有状态是 STATE_NONE，我们知道我们还没有启动蓝牙
            if (chatService.getState() == BluetoothChatService.STATE_NONE) {
                // 启动 BluetoothChat 服务
                chatService.start();
            }
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mHandler.removeCallbacksAndMessages(null);
        // 停止蓝牙通信连接服务
        if (chatService != null) chatService.stop();
    }

    @SuppressLint("InflateParams")
    private void initView() {
        LayoutInflater layoutInflater = LayoutInflater.from(this);
        View page_1, page_2, page_3;
        page_1 = layoutInflater.inflate(R.layout.page_1, null);
        page_2 = layoutInflater.inflate(R.layout.page_2, null);
        page_3 = layoutInflater.inflate(R.layout.page_3, null);
        views.add(page_1);
        views.add(page_2);
        views.add(page_3);
        viewPager.setAdapter(new MainPagerAdapter(views));

        mainTab_1_recycle = page_1.findViewById(R.id.mainTab_1_recycle);
        List<WaterfallStaggeredAdapter.ItemBean> itemBeanList = new ArrayList<>();
        itemBeanList.add(new WaterfallStaggeredAdapter.ItemBean("1", getResources().getDrawable(R.drawable.image_board_32), "监控调试层", "适用于通过蓝牙连接的基本STM32控制器 - HMI组件"));
        WaterfallStaggeredAdapter waterfallStaggeredAdapter = new WaterfallStaggeredAdapter(this, itemBeanList);
        waterfallStaggeredAdapter.setOnItemClickListener(this);
        mainTab_1_recycle.setAdapter(waterfallStaggeredAdapter);
        // new Stagger 布局管理器，布局管理器所需参数:1.规定显示几列 2.item 排列方向
        StaggeredGridLayoutManager staggeredGridLayoutManager = new StaggeredGridLayoutManager(2, StaggeredGridLayoutManager.VERTICAL);
        // 通过布局管理器控制条目排列的顺序  true:反向显示 false:正向显示
        staggeredGridLayoutManager.setReverseLayout(false);
        // 设置布局管理器，参数 StaggeredGridLayoutManager，可以是 RecyclerView 实现和 StaggeredGridView 一样的效果
        mainTab_1_recycle.setLayoutManager(staggeredGridLayoutManager);
    }

    private void setListener() {
        viewPager.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
            @Override
            public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {
            }

            @Override
            public void onPageSelected(int position) {
                switch (position) {
                    case 0:
                        navigation.setSelectedItemId(R.id.navigation_1);
                        break;
                    case 1:
                        navigation.setSelectedItemId(R.id.navigation_2);
                        break;
                    case 2:
                        navigation.setSelectedItemId(R.id.navigation_3);
                        break;
                }
            }

            @Override
            public void onPageScrollStateChanged(int state) {
            }
        });
        navigation.setOnNavigationItemSelectedListener(this);
    }

    private void loadData() {
    }

    public static void start(Context context) {
        Intent starter = new Intent(context, MainActivity.class);
        context.startActivity(starter);
    }

    private void showConnectDeviceProgress(boolean show) {
        if (dialog == null) {
            dialog = new ProgressDialog(this);
            dialog.setTitle("连接");
            dialog.setMessage("设备连接中······");
            dialog.setCancelable(false);
        }
        if (show) {
            dialog.show();
        } else {
            dialog.dismiss();
            dialog = null;
        }
    }

    @Override
    public void onItemClick(View view, int position) {
        switch ((String) view.getTag()) {
            case "1":
                BasicDeviceActivity.start(this);
                break;
            default:
        }
    }

    @OnClick({R.id.device})
    public void onViewClicked(View view) {
        switch (view.getId()) {
            case R.id.device:
                break;
            default:
        }
    }

    @Override
    public boolean onNavigationItemSelected(@NonNull MenuItem item) {
        switch (item.getItemId()) {
            case R.id.navigation_1:
                viewPager.setCurrentItem(0);
                icon.setText("{fa-server 20sp}");
                title.setText("设备");
                return true;
            case R.id.navigation_2:
                viewPager.setCurrentItem(1);
                icon.setText("{fa-pie-chart 20sp}");
                title.setText("数据");
                return true;
            case R.id.navigation_3:
                viewPager.setCurrentItem(2);
                icon.setText("{fa-cubes 20sp}");
                title.setText("扩展");
                return true;
        }
        return false;
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
        AlertDialog.Builder builder = new AlertDialog.Builder(MainActivity.this);
        builder.setTitle("退出")
                .setMessage("确定断开设备的蓝牙连接并退出程序吗？")
                .setNegativeButton("取消", null)
                .setPositiveButton("确定", (dialog, which) -> finish())
                .create()
                .show();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        switch (requestCode) {
            case 0:
                if (resultCode == RESULT_OK) {
                    // 初始化蓝牙连接服务
                    chatService = new BluetoothChatService(this, mHandler);
                    // 获取设备的 MAC 地址
                    String address = data.getExtras().getString(BluetoothChatService.EXTRA_DEVICE_ADDRESS);
                    // 尝试连接到设备
                    chatService.connect(bluetoothAdapter.getRemoteDevice(address));
                }
        }
    }

    @SuppressLint("HandlerLeak")
    private final Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case BluetoothChatService.MESSAGE_STATE_CHANGE:
                    System.err.println("MESSAGE_STATE_CHANGE = " + msg.arg1);
                    switch (msg.arg1) {
                        case BluetoothChatService.STATE_CONNECTED:
                            // 已连接
                            showConnectDeviceProgress(false);
                            break;
                        case BluetoothChatService.STATE_CONNECTING:
                            showConnectDeviceProgress(true);
                            break;
                        case BluetoothChatService.STATE_LISTEN:
                            showConnectDeviceProgress(false);
                            break;
                        case BluetoothChatService.STATE_NONE:
                            // 无连接
                            showConnectDeviceProgress(false);
                            break;
                    }
                    break;
                case BluetoothChatService.MESSAGE_READ:
                    byte[] readBuf = (byte[]) msg.obj;
                    String readMessage = null;
                    try {
                        readMessage = new String(readBuf, 0, msg.arg1, "GBK");
                    } catch (UnsupportedEncodingException e) {
                        e.printStackTrace();
                    }
                    System.err.println("/" + readMessage);

                    Pattern p = Pattern.compile("\\{.*?\\}");
                    Matcher m = p.matcher(readMessage);
                    // 判断正则表达式是否匹配到
                    if (m.find()) {
                        System.out.println("->" + m.group());
                    } else {
                        System.out.println("未找到");
                    }
                    //检错误码计算函数
//                    if (inhex == true) {
//                        String readMessage = " " + Data_syn.bytesToHexString(readBuf, msg.arg1);
//                        fmsg += readMessage;
//                        mConversationView.append(readMessage);
//                        // 接收计数，更显UI
//                        countin += readMessage.length() / 2;
//                        incount.setText("" + countin);
//                    } else if (inhex == false) {
//                    }
                    break;
                case BluetoothChatService.MESSAGE_DEVICE_NAME:
                    String connectedDeviceName = msg.getData().getString(BluetoothChatService.DEVICE_NAME);
                    Toast.makeText(getApplicationContext(), "连接到 " + connectedDeviceName, Toast.LENGTH_SHORT).show();
                    break;
                case BluetoothChatService.MESSAGE_TOAST:
                    Toast.makeText(getApplicationContext(), msg.getData().getString(BluetoothChatService.TOAST), Toast.LENGTH_SHORT).show();
                    break;
            }
        }
    };
}
