package cn.minbb.tester.activities;

import android.annotation.SuppressLint;
import android.app.ProgressDialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.annotation.NonNull;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.app.AppCompatDelegate;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.ListView;
import android.widget.Toast;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.tester.R;
import cn.minbb.tester.adapter.DeviceListAdapter;
import cn.minbb.tester.ctrl.App;
import cn.minbb.tester.ctrl.BluetoothChatService;

public class ScanDeviceActivity extends AppCompatActivity {

    @BindView(R.id.deviceList)
    ListView deviceList;

    private ProgressDialog dialog;

    private BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
    private List<DeviceListAdapter.ItemBean> deviceItemBeanList = new ArrayList<>();
    private DeviceListAdapter deviceListAdapter;
    private BluetoothDevice selectDevice;
    private int selectIndex;

    private BluetoothChatService chatService = null;

    static {
        AppCompatDelegate.setCompatVectorFromResourcesEnabled(true);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_scan_device);
        ButterKnife.bind(this);

        // 设置广播信息过滤
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(BluetoothDevice.ACTION_FOUND);
        intentFilter.addAction(BluetoothDevice.ACTION_BOND_STATE_CHANGED);
        intentFilter.addAction(BluetoothAdapter.ACTION_SCAN_MODE_CHANGED);
        intentFilter.addAction(BluetoothAdapter.ACTION_STATE_CHANGED);
        // 注册广播接收器，接收并处理搜索结果
        this.registerReceiver(receiver, intentFilter);

        // 检查设备是否支持蓝牙
        if (bluetoothAdapter == null) {
            // 设备不支持蓝牙
            Toast.makeText(ScanDeviceActivity.this, "设备不支持蓝牙", Toast.LENGTH_LONG).show();
            ScanDeviceActivity.this.finish();
        } else {
            if (!bluetoothAdapter.isEnabled()) {
                Intent intent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                // 设置蓝牙可见性，最多 300 秒
                intent.putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, 60);
                ScanDeviceActivity.this.startActivityForResult(intent, 0);
            }
            Set<BluetoothDevice> devices = bluetoothAdapter.getBondedDevices();
            for (BluetoothDevice device : devices) {
                deviceItemBeanList.add(new DeviceListAdapter.ItemBean(device));
            }
            deviceListAdapter = new DeviceListAdapter(this, deviceItemBeanList);
            deviceList.setAdapter(deviceListAdapter);
        }

        // 初始化蓝牙连接服务
        chatService = new BluetoothChatService(this, mHandler);

        initView();
        setListener();
        loadData();
    }

    @Override
    protected void onResume() {
        super.onResume();
//        if (chatService != null) {
//            // 只有状态是 STATE_NONE，我们知道我们还没有启动蓝牙
//            if (chatService.getState() == BluetoothChatService.STATE_NONE) {
//                // 启动 BluetoothChat 服务
//                chatService.start();
//            }
//        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        this.unregisterReceiver(receiver);
        mHandler.removeCallbacksAndMessages(null);
    }

    private void initView() {
        App.setupActionBar(this);
    }

    private void setListener() {
        deviceList.setOnItemClickListener((adapterView, view, i, l) -> {
            selectIndex = i;
            selectDevice = deviceItemBeanList.get(i).device;
            switch (selectDevice.getBondState()) {
                case BluetoothDevice.BOND_NONE:
                    // 未配对 - 尝试配对
                    try {
                        Method createBondMethod = BluetoothDevice.class.getMethod("createBond");
                        createBondMethod.invoke(selectDevice);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    break;
                case BluetoothDevice.BOND_BONDING:
                    // 配对中
                    break;
                case BluetoothDevice.BOND_BONDED:
                    // 已配对 - 尝试连接到设备
                    // BluetoothDevice device = bluetoothAdapter.getRemoteDevice(address);
                    // 设备的 MAC 地址
                    Intent intent = new Intent();
                    intent.putExtra(BluetoothChatService.EXTRA_DEVICE_ADDRESS, selectDevice.getAddress());
                    intent.putExtra(BluetoothChatService.EXTRA_LEGAL, false);
                    // 设置结果并完成此活动
                    setResult(RESULT_OK, intent);
                    finish();
                    break;
                default:
            }
        });
    }

    private void loadData() {
        // 请求定位权限
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (this.checkSelfPermission(android.Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                requestPermissions(new String[]{android.Manifest.permission.ACCESS_COARSE_LOCATION}, 1);
            }
        }
    }

    private void showSearchDeviceProgress(boolean show) {
        if (dialog == null) {
            dialog = new ProgressDialog(this);
            dialog.setTitle("蓝牙串口");
            dialog.setMessage("搜索设备中······");
            dialog.setCancelable(false);
            dialog.setButton(DialogInterface.BUTTON_NEGATIVE, "取消", (dialogInterface, i) -> {
                bluetoothAdapter.cancelDiscovery();
                dialog.dismiss();
            });
            dialog.setOnDismissListener(dialog -> {
                int count = 0;
                for (DeviceListAdapter.ItemBean itemBean : deviceItemBeanList) {
                    if (itemBean.device.getBondState() == BluetoothDevice.BOND_NONE) {
                        count++;
                    }
                }
                Toast.makeText(getApplicationContext(), "查找到 " + count + " 个蓝牙设备", Toast.LENGTH_SHORT).show();
            });
        }
        if (show) {
            dialog.show();
        } else {
            dialog.dismiss();
            dialog = null;
        }
    }

    private void showMatchDeviceProgress(boolean show) {
        if (dialog == null) {
            dialog = new ProgressDialog(this);
            dialog.setTitle("配对");
            dialog.setMessage("设备配对中······");
            dialog.setCancelable(false);
        }
        if (show) {
            dialog.show();
        } else {
            dialog.dismiss();
            dialog = null;
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        menu.add(0, 0, 0, "提示").setIcon(R.drawable.ic_info_outline_white_24dp).setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
        menu.add(0, 1, 0, "搜索设备").setIcon(R.drawable.ic_search_white_24dp).setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                ScanDeviceActivity.this.finish();
                break;
            case 0:
                AlertDialog.Builder builder = new AlertDialog.Builder(ScanDeviceActivity.this);
                builder.setIcon(R.drawable.ic_info_outline_black_24dp);
                builder.setTitle("提示");
                builder.setMessage("点击已配对的设备进行连接\n点击搜索到的设备进行配对");
                builder.setPositiveButton("好的", null);
                builder.create().show();
                break;
            case 1:
                showSearchDeviceProgress(true);
                List<DeviceListAdapter.ItemBean> unboundItemBeanList = new ArrayList<>();
                for (DeviceListAdapter.ItemBean itemBean : deviceItemBeanList) {
                    if (itemBean.device.getBondState() == BluetoothDevice.BOND_NONE) {
                        unboundItemBeanList.add(itemBean);
                    }
                }
                deviceItemBeanList.removeAll(unboundItemBeanList);
                deviceListAdapter.notifyDataSetChanged();
                // 打开蓝牙
                if (!bluetoothAdapter.isEnabled()) {
                    Intent intent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                    // 设置蓝牙可见性，最多 300 秒
                    intent.putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, 300);
                    ScanDeviceActivity.this.startActivity(intent);
                }
                // 寻找蓝牙设备，Android 会将查找到的设备以广播形式发出去
                if (!bluetoothAdapter.isDiscovering()) {
                    bluetoothAdapter.startDiscovery();
                }
                // 弹出是否允许扫描蓝牙设备的选择对话框
                // Intent intent = new Intent(BluetoothAdapter.ACTION_REQUEST_DISCOVERABLE);
                // startActivityForResult(intent, 1);
                new Handler().postDelayed(() -> runOnUiThread(() -> {
                    showSearchDeviceProgress(false);
                    bluetoothAdapter.cancelDiscovery();
                }), 20000);
                break;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        switch (requestCode) {
            case 0:
                if (resultCode == RESULT_CANCELED) {
                    ScanDeviceActivity.this.finish();
                    Toast.makeText(this, "蓝牙未打开", Toast.LENGTH_SHORT).show();
                }
                break;
            case 1:
                new Handler().postDelayed(() -> runOnUiThread(() -> {
                    // 刷新蓝牙设备列表
                    Toast.makeText(this, "刷新蓝牙设备列表", Toast.LENGTH_SHORT).show();
                }), 50);
                if (resultCode == RESULT_OK) {
                    Toast.makeText(this, "允许本地蓝牙被附近的其它蓝牙设备发现", Toast.LENGTH_SHORT).show();
                } else if (resultCode == RESULT_CANCELED) {
                    Toast.makeText(this, "不允许蓝牙被附近的其它蓝牙设备发现", Toast.LENGTH_SHORT).show();
                }
                break;
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        switch (requestCode) {
            case 1:
                if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    Toast.makeText(ScanDeviceActivity.this, "授权成功", Toast.LENGTH_SHORT).show();
                } else {
                    Toast.makeText(ScanDeviceActivity.this, "未获得授权，将无法进行扫描", Toast.LENGTH_SHORT).show();
                }
                break;
            default:
        }
    }

    @SuppressLint("HandlerLeak")
    private final Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case BluetoothChatService.MESSAGE_WRITE:
                    byte[] writeBuf = (byte[]) msg.obj;
                    // 自动发送
                    if (false) {
                        // 自动发送模块
                        mHandler.postDelayed(new Runnable() {
                            @Override
                            public void run() {
                            }
                        }, 1000);
                    } else if (false) {
                        mHandler.removeCallbacks(new Runnable() {
                            @Override
                            public void run() {
                            }
                        });
                    }
                    // 发送计数
//                    if (outhex == true) {
//                        String writeMessage = Data_syn.Bytes2HexString(writeBuf);
//                        countout += writeMessage.length() / 2;
//                        outcount.setText("" + countout);
//                    } else if (outhex == false) {
//                        String writeMessage = null;
//                        try {
//                            writeMessage = new String(writeBuf, "GBK");
//                        } catch (UnsupportedEncodingException e1) {
//                            e1.printStackTrace();
//                        }
//                        countout += writeMessage.length();
//                        outcount.setText("" + countout);
//                    }
                    break;
                case BluetoothChatService.MESSAGE_READ:
                    byte[] readBuf = (byte[]) msg.obj;
                    //检错误码计算函数
//                    if (inhex == true) {
//                        String readMessage = " " + Data_syn.bytesToHexString(readBuf, msg.arg1);
//                        fmsg += readMessage;
//                        mConversationView.append(readMessage);
//                        // 接收计数，更显UI
//                        countin += readMessage.length() / 2;
//                        incount.setText("" + countin);
//                    } else if (inhex == false) {
//                        String readMessage = null;
//                        try {
//                            readMessage = new String(readBuf, 0, msg.arg1, "GBK");
//                        } catch (UnsupportedEncodingException e) {
//                            e.printStackTrace();
//                        }
//                        fmsg += readMessage;
//                        mConversationView.append(readMessage);
//                        // 接收计数，更新UI
//                        countin += readMessage.length();
//                        incount.setText("" + countin);
//                    }
                    break;
                case BluetoothChatService.MESSAGE_DEVICE_NAME:
                    String connectedDeviceName = msg.getData().getString(BluetoothChatService.DEVICE_NAME);
                    Toast.makeText(getApplicationContext(), "连接到 " + connectedDeviceName, Toast.LENGTH_SHORT).show();
                    break;
                case BluetoothChatService.MESSAGE_TOAST:
                    Toast.makeText(getApplicationContext(), msg.getData().getString(BluetoothChatService.TOAST), Toast.LENGTH_SHORT).show();
                    break;
            }
        }
    };

    private BroadcastReceiver receiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            if (BluetoothDevice.ACTION_FOUND.equals(action)) {
                // 获取查找到的蓝牙设备
                BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                if (!deviceItemBeanList.contains(new DeviceListAdapter.ItemBean(device)) && device.getName() != null && device.getAddress() != null) {
                    deviceItemBeanList.add(new DeviceListAdapter.ItemBean(device));
                    deviceListAdapter.notifyDataSetChanged();
                }
                // 如果查找到的设备符合要连接的设备，处理
//                if (device.getName().equalsIgnoreCase(device.getName())) {
//                    // 搜索蓝牙设备的过程占用资源比较多，一旦找到需要连接的设备后需要及时关闭搜索
//                    App.getApp().getBluetoothAdapter().cancelDiscovery();
//                    // 获取蓝牙设备的连接状态
//                    switch (device.getBondState()) {
//                        // 未配对
//                        case BluetoothDevice.BOND_NONE:
//                            // 配对
//                            try {
//                                Method createBondMethod = BluetoothDevice.class.getMethod("createBond");
//                                createBondMethod.invoke(device);
//                            } catch (Exception e) {
//                                e.printStackTrace();
//                            }
//                            break;
//                        // 已配对
//                        case BluetoothDevice.BOND_BONDED:
//                            // 连接
//                            break;
//                    }
//                }
            } else if (BluetoothDevice.ACTION_BOND_STATE_CHANGED.equals(action)) {
                // 状态改变的广播
                BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                switch (device.getBondState()) {
                    case BluetoothDevice.BOND_NONE:
                        // 取消配对
                        System.out.println("取消配对");
                        showMatchDeviceProgress(false);
                        break;
                    case BluetoothDevice.BOND_BONDING:
                        // 正在配对
                        System.out.println("正在配对");
                        showMatchDeviceProgress(true);
                        break;
                    case BluetoothDevice.BOND_BONDED:
                        // 完成配对
                        System.out.println("完成配对");
                        showMatchDeviceProgress(false);
                        deviceItemBeanList.set(selectIndex, new DeviceListAdapter.ItemBean(device));
                        deviceListAdapter.notifyDataSetChanged();
                        Toast.makeText(getApplicationContext(), "和设备 " + device.getName() + " 配对成功", Toast.LENGTH_SHORT).show();
                        break;
                    default:
                }
            } else if (BluetoothAdapter.ACTION_STATE_CHANGED.equals(action)) {
                new Handler().postDelayed(() -> runOnUiThread(() -> {
                    if (!bluetoothAdapter.isEnabled()) {
                        ScanDeviceActivity.this.finish();
                    } else {
                        deviceItemBeanList.clear();
                        Set<BluetoothDevice> devices = bluetoothAdapter.getBondedDevices();
                        for (BluetoothDevice device : devices) {
                            deviceItemBeanList.add(new DeviceListAdapter.ItemBean(device));
                        }
                        deviceListAdapter.notifyDataSetChanged();
                    }
                }), 500);
            }
        }
    };
}
