package cn.minbb.tester.activities;

import android.annotation.SuppressLint;
import android.os.Bundle;
import android.os.Handler;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;

import java.util.Timer;
import java.util.TimerTask;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.tester.R;

// 通过用户交互显示和隐藏系统UI（即状态栏和导航/系统栏）的示例全屏活动。
public class StartActivity extends AppCompatActivity {

    @BindView(R.id.fullscreen_content)
    View contentView;
    @BindView(R.id.skip)
    Button skip;

    // 一些较旧的设备需要在UI小部件更新和状态和导航栏的更改之间稍微延迟。
    private Handler hideHandler = new Handler();
    private Runnable hideRunnable = new Runnable() {
        @SuppressLint("InlinedApi")
        @Override
        public void run() {
            // 延迟删除状态和导航栏
            // 请注意，其中一些常量是API 16（Jelly Bean）和API 19（KitKat）的新增内容。使用它们是安全的，因为它们在编译时被内联并且在早期设备上不执行任何操作。
            contentView.setSystemUiVisibility(View.SYSTEM_UI_FLAG_LOW_PROFILE
                    | View.SYSTEM_UI_FLAG_FULLSCREEN
                    | View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                    | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY
                    | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                    | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION);
        }
    };
    private Runnable showRunnable = () -> {
        // 延迟显示 UI 元素
        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.show();
        }
    };
    private Timer timer;
    private int time = 2;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_start);
        ButterKnife.bind(this);

        // 显示系统栏
        contentView.setSystemUiVisibility(View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION);
        // 延迟后显示 UI 元素
        hideHandler.removeCallbacks(hideRunnable);
        hideHandler.postDelayed(showRunnable, 300);


        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.hide();
        }
        // 延迟后删除状态和导航栏
        hideHandler.removeCallbacks(showRunnable);
        hideHandler.postDelayed(hideRunnable, 300);


        timer = new Timer();
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        if (time > 0) {
                            skip.setText(time-- + " s");
                        } else {
                            StartActivity.this.finish();
                            MainActivity.start(StartActivity.this);
                            overridePendingTransition(R.anim.main_activity_in, 0);
                        }
                    }
                });
            }
        }, 100, 1000);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        timer.cancel();
    }

    // 屏蔽返回键
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        return keyCode == KeyEvent.KEYCODE_BACK || super.onKeyDown(keyCode, event);
    }
}
