package cn.minbb.tester.adapter;

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.tester.R;

public class BasicDeviceAdapter extends RecyclerView.Adapter<BasicDeviceAdapter.ViewHolder> {

    // 映射数据
    private Context context;
    private List<ItemBean> itemBeen;

    private OnItemClickListener onItemClickListener;
    private OnItemLongClickListener onItemLongClickListener;

    public BasicDeviceAdapter(Context context, List<ItemBean> itemBeen) {
        this.context = context;
        this.itemBeen = itemBeen;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        // 转换一个 View 布局，决定了 item 的样子， 参数：1.上下文 2.Xml 布局资源 3.为 null
        View view = View.inflate(context, R.layout.item_activity_device_basic, null);
        // 创建一个 StaggerViewHolder 对象并把 ViewHolder 对象传出去
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        holder.keyText.setText(itemBeen.get(position).keyText);
        holder.valueText.setText(itemBeen.get(position).valueText);

        // 如果设置了回调，则设置点击事件
        if (onItemClickListener != null) {
            holder.itemView.setOnClickListener(v -> onItemClickListener.onItemClick(holder.itemView, holder.getLayoutPosition()));
        }

        if (onItemLongClickListener != null) {
            holder.itemView.setOnLongClickListener(v -> {
                onItemLongClickListener.onItemLongClick(holder.itemView, holder.getLayoutPosition());
                return false;
            });
        }
    }

    @Override
    public int getItemCount() {
        return itemBeen.size();
    }

    // ViewHolder 用于缓存控件
    class ViewHolder extends RecyclerView.ViewHolder {

        @BindView(R.id.key)
        TextView keyText;
        @BindView(R.id.value)
        TextView valueText;

        ViewHolder(View view) {
            super(view);
            ButterKnife.bind(this, view);
        }
    }

    public static class ItemBean {

        String keyText;
        String valueText;

        public ItemBean(String keyText, String valueText) {
            this.keyText = keyText;
            this.valueText = valueText;
        }
    }

    // 点击事件接口
    public interface OnItemClickListener {
        void onItemClick(View view, int position);
    }

    // 长按事件接口
    public interface OnItemLongClickListener {
        void onItemLongClick(View view, int position);
    }

    public void setOnItemClickListener(OnItemClickListener onItemClickListener) {
        this.onItemClickListener = onItemClickListener;
    }

    public void setOnItemLongClickListener(OnItemLongClickListener onItemLongClickListener) {
        this.onItemLongClickListener = onItemLongClickListener;
    }
}
