package cn.minbb.tester.adapter;

import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;

import com.joanzapata.iconify.widget.IconTextView;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.tester.R;
import lombok.Getter;
import lombok.Setter;

public class DeviceListAdapter extends BaseAdapter {

    // 映射数据
    private List<ItemBean> itemBeen;
    private LayoutInflater layoutInflater;

    public DeviceListAdapter(Context context, List<ItemBean> itemBeen) {
        layoutInflater = LayoutInflater.from(context);
        this.itemBeen = itemBeen;
    }

    @Override
    public int getCount() {
        // 获取数据量
        return itemBeen.size();
    }

    @Override
    public Object getItem(int position) {
        // 获取对应 ID 项对应的 Item
        return itemBeen.get(position);
    }

    @Override
    public long getItemId(int position) {
        // 获取对应项 ID
        return position;
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        ViewHolder viewHolder;
        if (convertView == null) {
            // 将 XML 转化为 View ，并不涉及到具体的布局，所以第二个参数通常设置为 null
            convertView = layoutInflater.inflate(R.layout.item_device_list, parent, false);
            viewHolder = new ViewHolder(convertView);
            convertView.setTag(viewHolder);
        } else {
            viewHolder = (ViewHolder) convertView.getTag();
        }
        // 取出 ItemBean 对象
        ItemBean itemBean = itemBeen.get(position);
        // 设置控件数据
        viewHolder.deviceName.setText(itemBean.deviceName);
        viewHolder.deviceAddress.setText(itemBean.deviceAddress);
        viewHolder.deviceStatus.setText(itemBean.deviceStatus);
        viewHolder.deviceTip.setText(itemBean.deviceTip);
        return convertView;
    }

    // ViewHolder 用于缓存控件
    class ViewHolder {

        @BindView(R.id.deviceName)
        TextView deviceName;
        @BindView(R.id.deviceAddress)
        TextView deviceAddress;
        @BindView(R.id.deviceStatus)
        IconTextView deviceStatus;
        @BindView(R.id.deviceTip)
        TextView deviceTip;

        ViewHolder(View view) {
            ButterKnife.bind(this, view);
        }
    }

    public static class ItemBean {

        @Getter
        @Setter
        public BluetoothDevice device;
        @Getter
        @Setter
        public String deviceName;
        @Getter
        @Setter
        public String deviceAddress;
        private String deviceStatus;
        private String deviceTip;

        public ItemBean(BluetoothDevice device) {
            this.device = device;
            this.deviceName = device.getName();
            this.deviceAddress = device.getAddress();
            switch (device.getBondState()) {
                case BluetoothDevice.BOND_NONE:
                    // 未配对
                    this.deviceStatus = "{fa-unlink}";
                    this.deviceTip = "点击配对";
                    break;
                case BluetoothDevice.BOND_BONDING:
                    // 配对中
                    break;
                case BluetoothDevice.BOND_BONDED:
                    // 已配对
                    this.deviceStatus = "{fa-link} 已配对";
                    this.deviceTip = "点击连接";
                    break;
                default:
            }
        }
    }
}
