package cn.minbb.tester.ctrl;

import android.annotation.SuppressLint;
import android.app.ActivityManager;
import android.app.Application;
import android.bluetooth.BluetoothDevice;
import android.content.ComponentName;
import android.content.Context;
import android.os.Environment;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;

import com.joanzapata.iconify.Iconify;
import com.joanzapata.iconify.fonts.FontAwesomeModule;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import cn.minbb.tester.R;
import lombok.Getter;
import lombok.Setter;

public class App extends Application {

    // 全局上下文
    private Context context = null;
    // 时间格式
    private static SimpleDateFormat simpleDateFormat = (SimpleDateFormat) SimpleDateFormat.getDateTimeInstance();

    @Getter
    @Setter
    private BluetoothDevice bluetoothDevice = null;

    public App() {
    }

    @SuppressLint("StaticFieldLeak")
    private static App app = null;

    public static App getApp() {
        return app;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        app = this;
        context = getApplicationContext();

        // 初始化字体图标模块
        Iconify.with(new FontAwesomeModule());
    }

    @Override
    public void onTerminate() {
        super.onTerminate();
    }

    // 获取系统当前时间
    public static String getCurrentStringTime() {
        return simpleDateFormat.format(getCurrentDateTime());
    }

    public static Date getCurrentDateTime() {
        return new Date();
    }

    public Context getContext() {
        return context;
    }

    // 获取 APP 文件夹路径
    public static String getAppDirectoryPath() {
        return Environment.getExternalStorageDirectory().getAbsolutePath() + File.separator + "Evaluation";
    }

    // 设置 ActionBar
    public static void setupActionBar(AppCompatActivity appCompatActivity) {
        // id from appcompat
        ViewGroup rootView = appCompatActivity.findViewById(R.id.action_bar_root);
        if (rootView != null) {
            View view = appCompatActivity.getLayoutInflater().inflate(R.layout.app_bar_layout, rootView, false);
            rootView.addView(view, 0);

            Toolbar toolbar = appCompatActivity.findViewById(R.id.toolbar);
            appCompatActivity.setSupportActionBar(toolbar);
        }

        ActionBar actionBar = appCompatActivity.getSupportActionBar();
        if (actionBar != null) {
            // 在 ActionBar 中显示 UP 按钮。
            actionBar.setDisplayHomeAsUpEnabled(true);
        }
    }

    public static boolean activityIsForeground(Context context, String className) {
        if (context == null || TextUtils.isEmpty(className)) {
            return false;
        }
        ActivityManager activityManager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        List<ActivityManager.RunningTaskInfo> list = activityManager.getRunningTasks(1);
        if (list != null && list.size() > 0) {
            ComponentName componentName = list.get(0).topActivity;
            return className.equals(componentName.getClassName());
        }
        return false;
    }
}
