package cn.minbb.tester.receiver;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.widget.Toast;

import java.lang.reflect.Method;

import cn.minbb.tester.ctrl.App;

public class BluetoothStatusReceiver extends BroadcastReceiver {
    @Override
    public void onReceive(Context context, Intent intent) {
        String action = intent.getAction();
        System.out.println("接收广播 action = " + action);
        if (BluetoothDevice.ACTION_FOUND.equals(action)) {
            // 获取查找到的蓝牙设备
            BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
            System.out.println("发现蓝牙设备 = " + device.getName());
            // 如果查找到的设备符合要连接的设备，处理
            if (device.getName().equalsIgnoreCase("STOP")) {
                // 搜索蓝牙设备的过程占用资源比较多，一旦找到需要连接的设备后需要及时关闭搜索
                BluetoothAdapter.getDefaultAdapter().cancelDiscovery();
                // 获取蓝牙设备的连接状态
                switch (device.getBondState()) {
                    // 未配对
                    case BluetoothDevice.BOND_NONE:
                        // 配对
                        try {
                            Method createBondMethod = BluetoothDevice.class.getMethod("createBond");
                            createBondMethod.invoke(device);
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                        break;
                    // 已配对
                    case BluetoothDevice.BOND_BONDED:
                        // 连接
                        break;
                }
            }
        } else if (BluetoothDevice.ACTION_BOND_STATE_CHANGED.equals(action)) {
            // 绑定状态改变的广播
            BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
            if (device.getName().equalsIgnoreCase(device.getName())) {
                switch (device.getBondState()) {
                    case BluetoothDevice.BOND_NONE:
                        // 取消配对
                        break;
                    case BluetoothDevice.BOND_BONDING:
                        // 正在配对
                        break;
                    case BluetoothDevice.BOND_BONDED:
                        // 完成配对
                        break;
                }
            }
        } else if (BluetoothDevice.ACTION_PAIRING_REQUEST.equals(action)) {
            // 配对请求
            Toast.makeText(App.getApp(), "接收到配对请求", Toast.LENGTH_SHORT).show();
        } else if (BluetoothAdapter.ACTION_STATE_CHANGED.equals(action)) {
            // 蓝牙状态更改
            if (!BluetoothAdapter.getDefaultAdapter().isEnabled()) {
                Toast.makeText(App.getApp(), "蓝牙已关闭", Toast.LENGTH_SHORT).show();
            }
        } else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED.equals(action)) {
            // 搜索结束
        }
    }
}
